@@ -43,10 +43,16 @@ def write_default_ansible_cfg(work_dir,
     config.set('defaults', 'retry_files_enabled', 'False')
     config.set('defaults', 'log_path',
                os.path.join(work_dir, 'ansible.log'))
+    config.set('defaults', 'forks', '25')
 
     # mistral user has no home dir set, so no place to save a known hosts file
     config.set('ssh_connection', 'ssh_args',
-               '-o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no')
+               '-o UserKnownHostsFile=/dev/null '
+               '-o StrictHostKeyChecking=no '
+               '-o ControlMaster=auto '
+               '-o ControlPersist=60s')
+    config.set('ssh_connection', 'control_path_dir',
+               os.path.join(work_dir, 'ansible-ssh'))
 
     with open(ansible_config_path, 'w') as configfile:
         config.write(configfile)
@@ -240,6 +246,9 @@ class AnsiblePlaybookAction(base.TripleOAction):
     def __init__(self, **kwargs):
         self._kwargs_for_run = kwargs
         self._playbook = self._kwargs_for_run.pop('playbook', None)
+        self.playbook_name = self._kwargs_for_run.pop('playbook_name',
+                                                      'playbook.yaml')
+        self.plan_name = self._kwargs_for_run.pop('plan_name', None)
         self.limit_hosts = self._kwargs_for_run.pop('limit_hosts', None)
         self.module_path = self._kwargs_for_run.pop('module_path', None)
         self.remote_user = self._kwargs_for_run.pop('remote_user', None)
@@ -277,6 +286,7 @@ class AnsiblePlaybookAction(base.TripleOAction):
         self.max_message_size = self._kwargs_for_run.pop(
             'max_message_size', 1048576)
         self.gather_facts = self._kwargs_for_run.pop('gather_facts', False)
+        self.trash_output = self._kwargs_for_run.pop('trash_output', False)
 
     @property
     def work_dir(self):
@@ -322,7 +332,7 @@ class AnsiblePlaybookAction(base.TripleOAction):
         elif not isinstance(self._playbook, six.string_types):
             self._playbook = yaml.safe_dump(self._playbook)
 
-        path = os.path.join(self.work_dir, 'playbook.yaml')
+        path = os.path.join(self.work_dir, self.playbook_name)
 
         # NOTE(flaper87):
         # We could probably catch parse errors here
@@ -360,10 +370,13 @@ class AnsiblePlaybookAction(base.TripleOAction):
         return path
 
     def format_message(self, message):
+        type_ = 'tripleo.ansible-playbook.{}'.format(self.playbook_name)
         return {
             'body': {
+                'type': type_,
                 'payload': {
                     'message': message,
+                    'plan_name': self.plan_name,
                     'status': 'RUNNING',
                     'execution': {'id': self.execution_id}}}}
 
@@ -461,11 +474,12 @@ class AnsiblePlaybookAction(base.TripleOAction):
                 env_variables.update(self.extra_env_variables)
 
             if self.use_openstack_credentials:
+                security_ctx = context.security
                 env_variables.update({
-                    'OS_AUTH_URL': context.auth_uri,
-                    'OS_USERNAME': context.user_name,
-                    'OS_AUTH_TOKEN': context.auth_token,
-                    'OS_PROJECT_NAME': context.project_name})
+                    'OS_AUTH_URL': security_ctx.auth_uri,
+                    'OS_USERNAME': security_ctx.user_name,
+                    'OS_AUTH_TOKEN': security_ctx.auth_token,
+                    'OS_PROJECT_NAME': security_ctx.project_name})
 
             command = [str(c) for c in command]
 
@@ -511,6 +525,9 @@ class AnsiblePlaybookAction(base.TripleOAction):
                 # stdout we don't know the difference. To keep the return dict
                 # similar there is an empty stderr. We can use the return code
                 # to determine if there was an error.
+                if self.trash_output:
+                    stdout = []
+                    stderr = ""
                 return {"stdout": "".join(stdout), "returncode": returncode,
                         "stderr": ""}
 
@@ -539,6 +556,8 @@ class AnsibleGenerateInventoryAction(base.TripleOAction):
             'work_dir', None)
         self.plan_name = self._kwargs_for_run.pop(
             'plan_name', 'overcloud')
+        self.ssh_network = self._kwargs_for_run.pop(
+            'ssh_network', 'ctlplane')
 
     @property
     def work_dir(self):
@@ -560,7 +579,8 @@ class AnsibleGenerateInventoryAction(base.TripleOAction):
             project_name=context.security.project_name,
             username=context.security.user_name,
             ansible_ssh_user=self.ansible_ssh_user,
-            plan_name=self.plan_name)
+            plan_name=self.plan_name,
+            host_network=self.ssh_network)
 
         inventory.write_static_inventory(inventory_path)
         return inventory_path
