@@ -44,6 +44,9 @@ DEFAULT_CONTAINER_NAME = 'overcloud'
 #: The default name to use for the config files of the container
 CONFIG_CONTAINER_NAME = 'overcloud-config'
 
+#: The default name to use for the container for validations
+VALIDATIONS_CONTAINER_NAME = 'tripleo-validations'
+
 #: The default key to use for updating parameters in plan environment.
 DEFAULT_PLAN_ENV_KEY = 'parameter_defaults'
 
@@ -145,6 +148,9 @@ DEFAULT_VOLUME_API_VERSION = '3'
 # import/export
 PLAN_ENVIRONMENT = 'plan-environment.yaml'
 
+# Name of the environment with merged parameters from CLI
+USER_ENVIRONMENT = 'user-environment.yaml'
+
 # The name of the file which holds container image default parameters
 CONTAINER_DEFAULTS_ENVIRONMENT = ('environments/'
                                   'containers-default-parameters.yaml')
@@ -171,3 +177,9 @@ TRIPLEO_DEPLOYMENT_RESOURCE = 'TripleODeployment'
 HOST_NETWORK = 'ctlplane'
 
 EXTERNAL_TASKS = ['external_deploy_tasks']
+
+ANSIBLE_ERRORS_FILE = 'ansible-errors.json'
+
+DEPLOYMENT_STATUS_FILE = 'deployment_status.yaml'
+
+MISTRAL_WORK_DIR = '/var/lib/mistral'
