@@ -21,10 +21,13 @@ import logging
 import netifaces
 import os
 import requests
+import shutil
 import six
 from six.moves import urllib
 import subprocess
+import tempfile
 import tenacity
+import yaml
 
 import docker
 try:
@@ -32,6 +35,7 @@ try:
 except ImportError:
     from docker import Client
 from oslo_concurrency import processutils
+from tripleo_common.actions import ansible
 from tripleo_common.image.base import BaseImageManager
 from tripleo_common.image.exception import ImageUploaderException
 
@@ -61,11 +65,13 @@ class ImageUploadManager(BaseImageManager):
        syntax. Multiple config files can be specified. They will be merged.
        """
 
-    def __init__(self, config_files=None, verbose=False, debug=False):
+    def __init__(self, config_files=None, verbose=False, debug=False,
+                 dry_run=False):
         if config_files is None:
             config_files = []
         super(ImageUploadManager, self).__init__(config_files)
         self.uploaders = {}
+        self.dry_run = dry_run
 
     def discover_image_tag(self, image, tag_from_label=None):
         uploader = self.uploader('docker')
@@ -96,9 +102,13 @@ class ImageUploadManager(BaseImageManager):
 
             # This updates the parsed upload_images dict with real values
             item['push_destination'] = push_destination
+            append_tag = item.get('modify_append_tag')
+            modify_role = item.get('modify_role')
+            modify_vars = item.get('modify_vars')
 
             self.uploader(uploader).add_upload_task(
-                image_name, pull_source, push_destination)
+                image_name, pull_source, push_destination,
+                append_tag, modify_role, modify_vars, self.dry_run)
 
         for uploader in self.uploaders.values():
             uploader.run_tasks()
@@ -122,7 +132,8 @@ class ImageUploader(object):
         pass
 
     @abc.abstractmethod
-    def add_upload_task(self, image_name, pull_source, push_destination):
+    def add_upload_task(self, image_name, pull_source, push_destination,
+                        append_tag, modify_role, modify_vars, dry_run):
         """Add an upload task to be executed later"""
         pass
 
@@ -150,41 +161,100 @@ class DockerImageUploader(ImageUploader):
         self.secure_registries = set(SECURE_REGISTRIES)
         self.insecure_registries = set()
 
+    @staticmethod
+    def run_modify_playbook(modify_role, modify_vars,
+                            source_image, target_image, append_tag):
+        vars = {}
+        if modify_vars:
+            vars.update(modify_vars)
+        vars['source_image'] = source_image
+        vars['target_image'] = target_image
+        vars['modified_append_tag'] = append_tag
+        LOG.debug('Playbook variables: \n%s' % yaml.safe_dump(
+            vars, default_flow_style=False))
+        playbook = [{
+            'hosts': 'localhost',
+            'tasks': [{
+                'name': 'Import role %s' % modify_role,
+                'import_role': {
+                    'name': modify_role
+                },
+                'vars': vars
+            }]
+        }]
+        LOG.debug('Playbook: \n%s' % yaml.safe_dump(
+            playbook, default_flow_style=False))
+        work_dir = tempfile.mkdtemp(prefix='tripleo-modify-image-playbook-')
+        try:
+            action = ansible.AnsiblePlaybookAction(
+                playbook=playbook,
+                work_dir=work_dir
+            )
+            result = action.run(None)
+            LOG.debug(result.get('stdout', ''))
+
+        finally:
+            shutil.rmtree(work_dir)
+
     @staticmethod
     def upload_image(image_name, pull_source, push_destination,
-                     insecure_registries):
+                     insecure_registries, append_tag, modify_role,
+                     modify_vars, dry_run):
         LOG.info('imagename: %s' % image_name)
-        dockerc = Client(base_url='unix://var/run/docker.sock', version='auto')
         if ':' in image_name:
             image = image_name.rpartition(':')[0]
-            tag = image_name.rpartition(':')[2]
+            source_tag = image_name.rpartition(':')[2]
         else:
             image = image_name
-            tag = 'latest'
+            source_tag = 'latest'
         if pull_source:
             repo = pull_source + '/' + image
         else:
             repo = image
 
-        full_image = repo + ':' + tag
-        new_repo = push_destination + '/' + repo.partition('/')[2]
-        full_new_repo = new_repo + ':' + tag
+        source_image = repo + ':' + source_tag
+        target_image_no_tag = push_destination + '/' + repo.partition('/')[2]
+        append_tag = append_tag or ''
+        target_tag = source_tag + append_tag
+        target_image_source_tag = target_image_no_tag + ':' + source_tag
+        target_image = target_image_no_tag + ':' + target_tag
 
-        if DockerImageUploader._images_match(full_image, full_new_repo,
-                                             insecure_registries):
-            LOG.info('Skipping upload for image %s' % image_name)
-            return []
+        if dry_run:
+            return source_image, target_image
 
-        DockerImageUploader._pull(dockerc, repo, tag=tag)
+        if modify_role:
+            if DockerImageUploader._image_exists(target_image,
+                                                 insecure_registries):
+                LOG.info('Skipping upload for modified image %s' %
+                         target_image)
+                return []
+        else:
+            if DockerImageUploader._images_match(source_image, target_image,
+                                                 insecure_registries):
+                LOG.info('Skipping upload for image %s' % image_name)
+                return []
 
-        response = dockerc.tag(image=full_image, repository=new_repo,
-                               tag=tag, force=True)
-        LOG.debug(response)
+        dockerc = Client(base_url='unix://var/run/docker.sock', version='auto')
+        DockerImageUploader._pull(dockerc, repo, tag=source_tag)
 
-        DockerImageUploader._push(dockerc, new_repo, tag=tag)
+        if modify_role:
+            DockerImageUploader.run_modify_playbook(
+                modify_role, modify_vars, source_image,
+                target_image_source_tag, append_tag)
+            # raise an exception if the playbook didn't tag
+            # the expected target image
+            dockerc.inspect_image(target_image)
+        else:
+            response = dockerc.tag(
+                image=source_image, repository=target_image_no_tag,
+                tag=target_tag, force=True
+            )
+            LOG.debug(response)
+
+        DockerImageUploader._push(dockerc, target_image_no_tag, tag=target_tag)
 
         LOG.info('Completed upload for image %s' % image_name)
-        return full_image, full_new_repo
+        return source_image, target_image
 
     @staticmethod
     @tenacity.retry(  # Retry up to 5 times with jittered exponential backoff
@@ -241,6 +311,15 @@ class DockerImageUploader(ImageUploader):
         i = DockerImageUploader._inspect(image_url.geturl(), insecure)
         return i.get('Digest')
 
+    @staticmethod
+    def _image_exists(image, insecure_registries):
+        try:
+            DockerImageUploader._image_digest(image, insecure_registries)
+        except Exception:
+            return False
+        else:
+            return True
+
     @staticmethod
     def _inspect(image, insecure=False):
 
@@ -353,7 +432,8 @@ class DockerImageUploader(ImageUploader):
                 else:
                     LOG.warning(e)
 
-    def add_upload_task(self, image_name, pull_source, push_destination):
+    def add_upload_task(self, image_name, pull_source, push_destination,
+                        append_tag, modify_role, modify_vars, dry_run):
         # prime self.insecure_registries
         if pull_source:
             self.is_insecure_registry(self._image_to_url(pull_source).netloc)
@@ -361,7 +441,8 @@ class DockerImageUploader(ImageUploader):
             self.is_insecure_registry(self._image_to_url(image_name).netloc)
         self.is_insecure_registry(self._image_to_url(push_destination).netloc)
         self.upload_tasks.append((image_name, pull_source, push_destination,
-                                  self.insecure_registries))
+                                  self.insecure_registries, append_tag,
+                                  modify_role, modify_vars, dry_run))
 
     def run_tasks(self):
         if not self.upload_tasks:
