@@ -21,6 +21,7 @@ import re
 import subprocess
 import sys
 import tempfile
+import time
 import yaml
 
 from tripleo_common.image import base
@@ -32,7 +33,7 @@ CONTAINER_IMAGE_PREPARE_PARAM = [{
         'namespace': 'docker.io/tripleomaster',
         'ceph_namespace': 'docker.io/ceph',
         'ceph_image': 'daemon',
-        'ceph_tag': 'v3.0.1-stable-3.0-luminous-centos-7-x86_64',
+        'ceph_tag': 'v3.0.3-stable-3.0-luminous-centos-7-x86_64',
         'name_prefix': 'centos-binary-',
         'name_suffix': '',
         'tag': 'current-tripleo',
@@ -73,8 +74,16 @@ def build_service_filter(environment, roles_data):
     :param roles_data: Roles file data used to filter services
     :returns: set of resource types representing containerized services
     """
+    if not roles_data:
+        return None
     enabled_services = get_enabled_services(environment, roles_data)
     containerized_services = set()
+    resource_registry = environment.get('resource_registry')
+    if not resource_registry:
+        # no way to tell which services are containerized, so just filter by
+        # enabled services
+        return enabled_services
+
     for service, env_path in environment.get('resource_registry', {}).items():
         # Use the template path to determine if it represents a
         # containerized service
@@ -84,7 +93,7 @@ def build_service_filter(environment, roles_data):
     return containerized_services.intersection(enabled_services)
 
 
-def container_images_prepare_multi(environment, roles_data):
+def container_images_prepare_multi(environment, roles_data, dry_run=False):
     """Perform multiple container image prepares and merge result
 
     Given the full heat environment and roles data, perform multiple image
@@ -111,9 +120,19 @@ def container_images_prepare_multi(environment, roles_data):
         mapping_args = cip_entry.get('set')
         push_destination = cip_entry.get('push_destination')
         pull_source = cip_entry.get('pull_source')
+        modify_role = cip_entry.get('modify_role')
+        modify_vars = cip_entry.get('modify_vars')
+        if modify_role:
+            append_tag = cip_entry.get(
+                'modify_append_tag',
+                time.strftime('-modified-%Y%m%d%H%M%S')
+            )
+        else:
+            append_tag = None
 
         prepare_data = container_images_prepare(
             excludes=cip_entry.get('excludes'),
+            includes=cip_entry.get('includes'),
             service_filter=service_filter,
             pull_source=pull_source,
             push_destination=push_destination,
@@ -121,16 +140,22 @@ def container_images_prepare_multi(environment, roles_data):
             output_env_file='image_params',
             output_images_file='upload_data',
             tag_from_label=cip_entry.get('tag_from_label'),
+            append_tag=append_tag,
+            modify_role=modify_role,
+            modify_vars=modify_vars,
         )
         env_params.update(prepare_data['image_params'])
 
-        if push_destination or pull_source:
+        if push_destination or pull_source or modify_role:
             with tempfile.NamedTemporaryFile(mode='w') as f:
                 yaml.safe_dump({
                     'container_images': prepare_data['upload_data']
                 }, f)
                 uploader = image_uploader.ImageUploadManager(
-                    [f.name], verbose=True)
+                    [f.name],
+                    verbose=True,
+                    dry_run=dry_run,
+                )
                 uploader.upload()
     return env_params
 
@@ -146,14 +171,18 @@ def container_images_prepare_defaults():
 
 
 def container_images_prepare(template_file=DEFAULT_TEMPLATE_FILE,
-                             excludes=None, service_filter=None,
+                             excludes=None, includes=None, service_filter=None,
                              pull_source=None, push_destination=None,
                              mapping_args=None, output_env_file=None,
-                             output_images_file=None, tag_from_label=None):
+                             output_images_file=None, tag_from_label=None,
+                             append_tag=None, modify_role=None,
+                             modify_vars=None):
     """Perform container image preparation
 
     :param template_file: path to Jinja2 file containing all image entries
     :param excludes: list of image name substrings to use for exclude filter
+    :param includes: list of image name substrings, at least one must match.
+                     All excludes are ignored if includes is specified.
     :param service_filter: set of heat resource types for containerized
                            services to filter by. Disable by passing None.
     :param pull_source: DEPRECATED namespace for pulling during image uploads
@@ -166,12 +195,18 @@ def container_images_prepare(template_file=DEFAULT_TEMPLATE_FILE,
     :param output_images_file: key to use for image upload data
     :param tag_from_label: string when set will trigger tag discovery on every
                            image
+    :param append_tag: string to append to the tag for the destination image
+    :param modify_role: string of ansible role name to run during upload before
+                        the push to destination
+    :param modify_vars: dict of variables to pass to modify_role
     :returns: dict with entries for the supplied output_env_file or
               output_images_file
     """
 
     if mapping_args is None:
         mapping_args = {}
+    if not append_tag:
+        append_tag = ''
 
     if service_filter:
         if 'OS::TripleO::Services::OpenDaylightApi' in service_filter:
@@ -181,15 +216,20 @@ def container_images_prepare(template_file=DEFAULT_TEMPLATE_FILE,
 
     def ffunc(entry):
         imagename = entry.get('imagename', '')
-        if excludes:
-            for p in excludes:
-                if re.search(p, imagename):
-                    return None
         if service_filter is not None:
             # check the entry is for a service being deployed
             image_services = set(entry.get('services', []))
             if not image_services.intersection(service_filter):
                 return None
+        if includes:
+            for p in includes:
+                if re.search(p, imagename):
+                    return entry
+            return None
+        if excludes:
+            for p in excludes:
+                if re.search(p, imagename):
+                    return None
         return entry
 
     builder = KollaImageBuilder([template_file])
@@ -219,9 +259,15 @@ def container_images_prepare(template_file=DEFAULT_TEMPLATE_FILE,
             # push_destination, since that is where they will be uploaded to
             image = imagename.partition('/')[2]
             imagename = '/'.join((push_destination, image))
+        if append_tag:
+            entry['append_tag'] = append_tag
+        if modify_role:
+            entry['modify_role'] = modify_role
+        if modify_vars:
+            entry['modify_vars'] = modify_vars
         if 'params' in entry:
             for p in entry.pop('params'):
-                params[p] = imagename
+                params[p] = imagename + append_tag
         if 'services' in entry:
             del(entry['services'])
 
