@@ -85,10 +85,12 @@ class StackOutputs(object):
 class TripleoInventory(object):
     def __init__(self, configs=None, session=None, hclient=None,
                  plan_name=None, auth_url=None, project_name=None,
-                 cacert=None, username=None, ansible_ssh_user=None):
+                 cacert=None, username=None, ansible_ssh_user=None,
+                 host_network=None):
         self.session = session
         self.hclient = hclient
         self.hosts_format_dict = False
+        self.host_network = host_network or HOST_NETWORK
         if configs is not None:
             # FIXME(shardy) backwards compatibility until we switch
             # tripleo-validations to pass the individual values
@@ -106,6 +108,7 @@ class TripleoInventory(object):
             self.ansible_ssh_user = ansible_ssh_user
             self.plan_name = plan_name
         self.stack_outputs = StackOutputs(self.plan_name, self.hclient)
+        self.hostvars = {}
 
     @staticmethod
     def get_roles_by_service(enabled_services):
@@ -152,10 +155,9 @@ class TripleoInventory(object):
     def list(self):
         ret = OrderedDict({
             'Undercloud': {
-                'hosts': {
-                    'undercloud': {
-                        'ansible_host': 'localhost'}},
+                'hosts': self._hosts(['undercloud']),
                 'vars': {
+                    'ansible_host': 'localhost',
                     'ansible_connection': 'local',
                     'auth_url': self.auth_url,
                     'cacert': self.cacert,
@@ -201,9 +203,10 @@ class TripleoInventory(object):
         children = []
         for role, hostnames in role_net_hostname_map.items():
             if hostnames:
-                names = hostnames.get(HOST_NETWORK) or []
-                shortnames = [n.split(".%s." % HOST_NETWORK)[0] for n in names]
-                ips = role_net_ip_map[role][HOST_NETWORK]
+                names = hostnames.get(self.host_network) or []
+                shortnames = [n.split(".%s." % self.host_network)[0]
+                              for n in names]
+                ips = role_net_ip_map[role][self.host_network]
                 hosts = {}
                 for idx, name in enumerate(shortnames):
                     hosts[name] = {}
@@ -225,16 +228,26 @@ class TripleoInventory(object):
                     networks.update(hosts[name]['enabled_networks'])
 
                 children.append(role)
+
+                if self.hosts_format_dict:
+                    hosts_format = hosts
+                else:
+                    hosts_format = [h for h in hosts.keys()]
+                    hosts_format.sort()
+
                 ret[role] = {
-                    'hosts': hosts,
+                    'hosts': hosts_format,
                     'vars': {
                         'ansible_ssh_user': self.ansible_ssh_user,
                         'bootstrap_server_id': role_node_id_map.get(
                             'bootstrap_server_id'),
-                        'role_name': role,
+                        'tripleo_role_name': role,
                     }
+
                 }
 
+                self.hostvars.update(hosts)
+
         if children:
             vip_map = self.stack_outputs.get('VipMap', {})
             vips = {(vip_name + "_vip"): vip
@@ -259,8 +272,10 @@ class TripleoInventory(object):
                     }
                 }
 
-        # Prevent Ansible from repeatedly calling us to get empty host details
-        ret['_meta'] = {'hostvars': {}}
+        if not self.hosts_format_dict:
+            # Prevent Ansible from repeatedly calling us to get empty host
+            # details
+            ret['_meta'] = {'hostvars': self.hostvars}
 
         return ret
 
