@@ -55,7 +55,6 @@ class AnsibleActionTest(base.TestCase):
 
         env = {
             'HOME': action.work_dir,
-            'ANSIBLE_LOCAL_TEMP': action.work_dir,
             'ANSIBLE_CONFIG': ansible_config_path
         }
 
@@ -102,10 +101,7 @@ class AnsiblePlaybookActionTest(base.TestCase):
         pb = os.path.join(action.work_dir, 'playbook.yaml')
         env = {
             'HOME': action.work_dir,
-            'ANSIBLE_LOCAL_TEMP': action.work_dir,
-            'ANSIBLE_CONFIG': ansible_config_path,
-            'ANSIBLE_CALLBACK_WHITELIST': 'profile_tasks',
-            'PROFILE_TASKS_TASK_OUTPUT_LIMIT': '0',
+            'ANSIBLE_CONFIG': ansible_config_path
         }
 
         mock_execute.assert_called_once_with(
