@@ -167,7 +167,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'namespace': 'docker.io/tripleomaster',
                 'ceph_namespace': 'docker.io/ceph',
                 'ceph_image': 'daemon',
-                'ceph_tag': 'v3.0.1-stable-3.0-luminous-centos-7-x86_64',
+                'ceph_tag': 'v3.0.3-stable-3.0-luminous-centos-7-x86_64',
                 'name_prefix': 'centos-binary-',
                 'name_suffix': '',
                 'tag': 'current-tripleo',
@@ -401,6 +401,54 @@ class TestPrepare(base.TestCase):
             )
         )
 
+    @mock.patch('requests.get')
+    def test_prepare_includes(self, mock_get):
+        self.assertEqual({
+            'container_images.yaml': [
+                {'imagename': '192.0.2.0:8787/t/p-nova-libvirt:l'}
+            ],
+            'environments/containers-default-parameters.yaml': {
+                'DockerNovaLibvirtImage': '192.0.2.0:8787/t/p-nova-libvirt:l'
+            }},
+            kb.container_images_prepare(
+                template_file=TEMPLATE_PATH,
+                output_env_file=constants.CONTAINER_DEFAULTS_ENVIRONMENT,
+                output_images_file='container_images.yaml',
+                includes=['libvirt'],
+                mapping_args={
+                    'namespace': '192.0.2.0:8787/t',
+                    'name_prefix': 'p',
+                    'name_suffix': '',
+                    'tag': 'l',
+                }
+            )
+        )
+
+    @mock.patch('requests.get')
+    def test_prepare_includes_excludes(self, mock_get):
+        # assert same result as includes only. includes trumps excludes
+        self.assertEqual({
+            'container_images.yaml': [
+                {'imagename': '192.0.2.0:8787/t/p-nova-libvirt:l'}
+            ],
+            'environments/containers-default-parameters.yaml': {
+                'DockerNovaLibvirtImage': '192.0.2.0:8787/t/p-nova-libvirt:l'
+            }},
+            kb.container_images_prepare(
+                template_file=TEMPLATE_PATH,
+                output_env_file=constants.CONTAINER_DEFAULTS_ENVIRONMENT,
+                output_images_file='container_images.yaml',
+                includes=['libvirt'],
+                excludes=['libvirt'],
+                mapping_args={
+                    'namespace': '192.0.2.0:8787/t',
+                    'name_prefix': 'p',
+                    'name_suffix': '',
+                    'tag': 'l',
+                }
+            )
+        )
+
     @mock.patch('requests.get')
     def test_prepare_push_dest(self, mock_get):
         self.assertEqual({
@@ -665,11 +713,14 @@ class TestPrepare(base.TestCase):
                 'ContainerImagePrepare': [{
                     'set': mapping_args,
                     'tag_from_label': 'foo',
+                    'includes': ['nova', 'neutron'],
                 }, {
                     'set': mapping_args,
                     'tag_from_label': 'bar',
                     'excludes': ['nova', 'neutron'],
-                    'push_destination': '192.0.2.1:8787'
+                    'push_destination': '192.0.2.1:8787',
+                    'modify_role': 'add-foo-plugin',
+                    'modify_vars': {'foo_version': '1.0.1'}
                 }]
             }
         }
@@ -703,23 +754,31 @@ class TestPrepare(base.TestCase):
         mock_cip.assert_has_calls([
             mock.call(
                 excludes=None,
+                includes=['nova', 'neutron'],
                 mapping_args=mapping_args,
                 output_env_file='image_params',
                 output_images_file='upload_data',
                 pull_source=None,
                 push_destination=None,
-                service_filter=set([]),
-                tag_from_label='foo'
+                service_filter=None,
+                tag_from_label='foo',
+                append_tag=None,
+                modify_role=None,
+                modify_vars=None
             ),
             mock.call(
                 excludes=['nova', 'neutron'],
+                includes=None,
                 mapping_args=mapping_args,
                 output_env_file='image_params',
                 output_images_file='upload_data',
                 pull_source=None,
                 push_destination='192.0.2.1:8787',
-                service_filter=set([]),
-                tag_from_label='bar'
+                service_filter=None,
+                tag_from_label='bar',
+                append_tag=mock.ANY,
+                modify_role='add-foo-plugin',
+                modify_vars={'foo_version': '1.0.1'}
             )
         ])
 
@@ -734,3 +793,99 @@ class TestPrepare(base.TestCase):
             },
             image_params
         )
+
+    @mock.patch('tripleo_common.image.kolla_builder.container_images_prepare')
+    @mock.patch('tripleo_common.image.image_uploader.ImageUploadManager',
+                autospec=True)
+    def test_container_images_prepare_multi_dry_run(self, mock_im, mock_cip):
+        mapping_args = {
+            'namespace': 't',
+            'name_prefix': '',
+            'name_suffix': '',
+            'tag': 'l',
+        }
+        env = {
+            'parameter_defaults': {
+                'ContainerImagePrepare': [{
+                    'set': mapping_args,
+                    'tag_from_label': 'foo',
+                }, {
+                    'set': mapping_args,
+                    'tag_from_label': 'bar',
+                    'excludes': ['nova', 'neutron'],
+                    'push_destination': '192.0.2.1:8787',
+                    'modify_role': 'add-foo-plugin',
+                    'modify_vars': {'foo_version': '1.0.1'},
+                    'modify_append_tag': 'modify-123'
+                }]
+            }
+        }
+        roles_data = []
+        mock_cip.side_effect = [
+            {
+                'image_params': {
+                    'FooImage': 't/foo:latest',
+                    'BarImage': 't/bar:latest',
+                    'BazImage': 't/baz:latest',
+                    'BinkImage': 't/bink:latest'
+                },
+                'upload_data': []
+            }, {
+                'image_params': {
+                    'BarImage': 't/bar:1.0',
+                    'BazImage': 't/baz:1.0'
+                },
+                'upload_data': [{
+                    'imagename': 't/bar:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }, {
+                    'imagename': 't/baz:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }]
+            },
+        ]
+
+        image_params = kb.container_images_prepare_multi(env, roles_data, True)
+
+        mock_cip.assert_has_calls([
+            mock.call(
+                excludes=None,
+                includes=None,
+                mapping_args=mapping_args,
+                output_env_file='image_params',
+                output_images_file='upload_data',
+                pull_source=None,
+                push_destination=None,
+                service_filter=None,
+                tag_from_label='foo',
+                append_tag=None,
+                modify_role=None,
+                modify_vars=None
+            ),
+            mock.call(
+                excludes=['nova', 'neutron'],
+                includes=None,
+                mapping_args=mapping_args,
+                output_env_file='image_params',
+                output_images_file='upload_data',
+                pull_source=None,
+                push_destination='192.0.2.1:8787',
+                service_filter=None,
+                tag_from_label='bar',
+                append_tag=mock.ANY,
+                modify_role='add-foo-plugin',
+                modify_vars={'foo_version': '1.0.1'}
+            )
+        ])
+
+        mock_im.assert_called_once_with(mock.ANY, dry_run=True, verbose=True)
+
+        self.assertEqual(
+            {
+                'BarImage': 't/bar:1.0',
+                'BazImage': 't/baz:1.0',
+                'BinkImage': 't/bink:latest',
+                'FooImage': 't/foo:latest'
+            },
+            image_params
+        )
