@@ -258,9 +258,9 @@ class NodesTest(base.TestCase):
 
     def _get_node(self):
         return {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'amd64',
-                'mac': ['aaa'], 'pm_addr': 'foo.bar', 'pm_user': 'test',
-                'pm_password': 'random', 'pm_type': 'ipmi', 'name': 'node1',
-                'capabilities': 'num_nics:6'}
+                'ports': [{'address': 'aaa'}], 'pm_addr': 'foo.bar',
+                'pm_user': 'test', 'pm_password': 'random', 'pm_type': 'ipmi',
+                'name': 'node1', 'capabilities': 'num_nics:6'}
 
     def test_register_all_nodes_ironic_no_hw_stats(self):
         node_list = [self._get_node()]
@@ -287,7 +287,8 @@ class NodesTest(base.TestCase):
                              resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', physical_network='ctlplane',
+                              local_link_connection=None)
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -311,7 +312,8 @@ class NodesTest(base.TestCase):
                              resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', physical_network='ctlplane',
+                              local_link_connection=None)
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -334,14 +336,17 @@ class NodesTest(base.TestCase):
                                 "ipmi_username": "test",
                                 "ipmi_password": "random",
                                 "deploy_kernel": "kernel-123",
-                                "deploy_ramdisk": "ramdisk-999"}
+                                "deploy_ramdisk": "ramdisk-999",
+                                "rescue_kernel": "kernel-123",
+                                "rescue_ramdisk": "ramdisk-999"}
         pxe_node = mock.call(driver="ipmi",
                              name='node1',
                              driver_info=pxe_node_driver_info,
                              resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', physical_network='ctlplane',
+                              local_link_connection=None)
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -365,7 +370,8 @@ class NodesTest(base.TestCase):
                              resource_class='baremetal',
                              uuid="abcdef")
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', physical_network='ctlplane',
+                              local_link_connection=None)
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -390,7 +396,8 @@ class NodesTest(base.TestCase):
                              resource_class='baremetal',
                              properties=node_properties)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', physical_network='ctlplane',
+                              local_link_connection=None)
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -403,6 +410,7 @@ class NodesTest(base.TestCase):
                       'network_interface': 'neutron',
                       'power_interface': 'ipmitool',
                       'raid_interface': 'agent',
+                      'rescue_interface': 'agent',
                       'storage_interface': 'cinder',
                       'vendor_interface': 'ipmitool'}
 
@@ -425,7 +433,8 @@ class NodesTest(base.TestCase):
                              resource_class='baremetal',
                              **interfaces)
         port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
-                              address='aaa', physical_network='ctlplane')
+                              address='aaa', local_link_connection=None,
+                              physical_network='ctlplane')
         ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
         ironic.port.create.assert_has_calls([port_call])
 
@@ -438,6 +447,7 @@ class NodesTest(base.TestCase):
                       'network_interface': 'neutron',
                       'power_interface': 'ipmitool',
                       'raid_interface': 'agent',
+                      'rescue_interface': 'agent',
                       'storage_interface': 'cinder',
                       'vendor_interface': 'ipmitool'}
 
@@ -492,6 +502,8 @@ class NodesTest(base.TestCase):
                 {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                 {'path': '/driver_info/deploy_kernel', 'value': 'image-k'},
                 {'path': '/driver_info/deploy_ramdisk', 'value': 'image-r'},
+                {'path': '/driver_info/rescue_kernel', 'value': 'image-k'},
+                {'path': '/driver_info/rescue_ramdisk', 'value': 'image-r'},
                 {'path': '/driver_info/ipmi_username', 'value': 'test'}]
             for key in update_patch:
                 key['op'] = 'add'
@@ -589,7 +601,7 @@ class NodesTest(base.TestCase):
 
     def test_register_node_update(self):
         node = self._get_node()
-        node['mac'][0] = node['mac'][0].upper()
+        node['ports'][0]['address'] = node['ports'][0]['address'].upper()
         ironic = mock.MagicMock()
         node_map = {'mac': {'aaa': 1}}
 
@@ -655,12 +667,29 @@ class NodesTest(base.TestCase):
         node['pm_type'] = 'fake_pxe'
         client = mock.MagicMock()
         nodes.register_ironic_node(node, client=client)
-        client.node.create.assert_called_once_with(driver='fake_pxe',
+        client.node.create.assert_called_once_with(driver='manual-management',
                                                    name='node1',
                                                    properties=node_properties,
                                                    resource_class='baremetal',
                                                    driver_info={})
 
+    def test_register_ironic_node_ucs(self):
+        node_properties = {"cpus": "1",
+                           "memory_mb": "2048",
+                           "local_gb": "30",
+                           "cpu_arch": "amd64",
+                           "capabilities": "num_nics:6"}
+        node = self._get_node()
+        node['pm_type'] = 'cisco-ucs-managed'
+        client = mock.MagicMock()
+        nodes.register_ironic_node(node, client=client)
+        client.node.create.assert_called_once_with(
+            driver='cisco-ucs-managed', name='node1',
+            properties=node_properties,
+            resource_class='baremetal',
+            driver_info={'ucs_password': 'random', 'ucs_address': 'foo.bar',
+                         'ucs_username': 'test'})
+
     def test_register_ironic_node_pxe_ucs(self):
         node_properties = {"cpus": "1",
                            "memory_mb": "2048",
@@ -672,7 +701,8 @@ class NodesTest(base.TestCase):
         client = mock.MagicMock()
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
-            driver='pxe_ucs', name='node1', properties=node_properties,
+            driver='cisco-ucs-managed', name='node1',
+            properties=node_properties,
             resource_class='baremetal',
             driver_info={'ucs_password': 'random', 'ucs_address': 'foo.bar',
                          'ucs_username': 'test'})
@@ -706,7 +736,7 @@ class NodesTest(base.TestCase):
         client = mock.MagicMock()
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
-            driver='pxe_ipmitool', name='node1', properties=node_properties,
+            driver='ipmi', name='node1', properties=node_properties,
             resource_class='baremetal',
             driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                          'ipmi_username': 'test', 'ipmi_port': '6230'})
@@ -757,11 +787,28 @@ class NodesTest(base.TestCase):
         client = mock.MagicMock()
         nodes.register_ironic_node(node, client=client)
         client.node.create.assert_called_once_with(
-            driver='pxe_drac', name='node1', properties=node_properties,
+            driver='idrac', name='node1', properties=node_properties,
             resource_class='baremetal',
             driver_info={'drac_password': 'random', 'drac_address': 'foo.bar',
                          'drac_username': 'test', 'drac_port': '6230'})
 
+    def test_register_ironic_node_pxe_ilo(self):
+        node_properties = {"cpus": "1",
+                           "memory_mb": "2048",
+                           "local_gb": "30",
+                           "cpu_arch": "amd64",
+                           "capabilities": "num_nics:6"}
+        node = self._get_node()
+        node['pm_type'] = 'pxe_ilo'
+        node['pm_port'] = '1234'
+        client = mock.MagicMock()
+        nodes.register_ironic_node(node, client=client)
+        client.node.create.assert_called_once_with(
+            driver='ilo', name='node1', properties=node_properties,
+            resource_class='baremetal',
+            driver_info={'ilo_password': 'random', 'ilo_address': 'foo.bar',
+                         'ilo_username': 'test', 'ilo_port': '1234'})
+
     def test_register_ironic_node_redfish(self):
         node_properties = {"cpus": "1",
                            "memory_mb": "2048",
@@ -781,6 +828,38 @@ class NodesTest(base.TestCase):
                          'redfish_username': 'test',
                          'redfish_system_id': '/redfish/v1/Systems/1'})
 
+    def test_register_ironic_node_with_physical_network(self):
+        node = self._get_node()
+        node['ports'] = [{'physical_network': 'subnet1', 'address': 'aaa'}]
+        ironic = mock.MagicMock()
+        nodes.register_ironic_node(node, client=ironic)
+        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
+                              address='aaa', physical_network='subnet1',
+                              local_link_connection=None)
+        ironic.port.create.assert_has_calls([port_call])
+
+    def test_register_ironic_node_with_local_link_connection(self):
+        node = self._get_node()
+        node['ports'] = [
+            {
+                'local_link_connection': {
+                    "switch_info": "switch",
+                    "port_id": "port1",
+                    "switch_id": "bbb"
+                },
+                'physical_network': 'subnet1',
+                'address': 'aaa'
+            }
+        ]
+        ironic = mock.MagicMock()
+        nodes.register_ironic_node(node, client=ironic)
+        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
+                              address='aaa', physical_network='subnet1',
+                              local_link_connection={"switch_info": "switch",
+                                                     "port_id": "port1",
+                                                     "switch_id": "bbb"})
+        ironic.port.create.assert_has_calls([port_call])
+
     def test_clean_up_extra_nodes_ironic(self):
         node = collections.namedtuple('node', ['uuid'])
         client = mock.MagicMock()
@@ -789,16 +868,16 @@ class NodesTest(base.TestCase):
         nodes._clean_up_extra_nodes(seen, client, remove=True)
         client.node.delete.assert_called_once_with('foobar')
 
-    def test__get_node_id_fake_pxe(self):
+    def test__get_node_id_manual_management(self):
         node = self._get_node()
-        node['pm_type'] = 'fake_pxe'
-        handler = nodes.find_driver_handler('fake_pxe')
+        node['pm_type'] = 'manual-management'
+        handler = nodes.find_driver_handler('manual-management')
         node_map = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {}}
         self.assertEqual('abcdef', nodes._get_node_id(node, handler, node_map))
 
     def test__get_node_id_conflict(self):
         node = self._get_node()
-        handler = nodes.find_driver_handler('pxe_ipmitool')
+        handler = nodes.find_driver_handler('ipmi')
         node_map = {'mac': {'aaa': 'abcdef'},
                     'pm_addr': {'foo.bar': 'defabc'}}
         self.assertRaises(exception.InvalidNode,
@@ -807,7 +886,7 @@ class NodesTest(base.TestCase):
 
     def test_get_node_id_valid_duplicate(self):
         node = self._get_node()
-        handler = nodes.find_driver_handler('pxe_ipmitool')
+        handler = nodes.find_driver_handler('ipmi')
         node_map = {'mac': {'aaa': 'id'},
                     'pm_addr': {'foo.bar': 'id'}}
         self.assertEqual('id', nodes._get_node_id(node, handler, node_map))
@@ -830,12 +909,12 @@ class TestPopulateNodeMapping(base.TestCase):
                     'uuids': {'abcdef', 'fedcba', 'xyz'}}
         self.assertEqual(expected, nodes._populate_node_mapping(client))
 
-    def test_populate_node_mapping_ironic_fake_pxe(self):
+    def test_populate_node_mapping_ironic_manual_management(self):
         client = mock.MagicMock()
         ironic_node = collections.namedtuple('node', ['uuid', 'driver',
                                              'driver_info'])
         ironic_port = collections.namedtuple('port', ['address'])
-        node = ironic_node('abcdef', 'fake_pxe', None)
+        node = ironic_node('abcdef', 'manual-management', None)
         client.node.list_ports.return_value = [ironic_port('aaa')]
         client.node.list.return_value = [node]
         expected = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {},
@@ -859,8 +938,10 @@ VALID_NODE_JSON = [
      'pm_password': 'p@$$w0rd',
      'pm_port': 1234,
      'ipmi_priv_level': 'USER',
-     'mac': ['aa:bb:cc:dd:ee:ff',
-             '11:22:33:44:55:66'],
+     'ports': [
+         {'address': 'aa:bb:cc:dd:ee:ff'},
+         {'address': '11:22:33:44:55:66'}
+     ],
      'name': 'foobar1',
      'capabilities': {'foo': 'bar'},
      'kernel_id': 'kernel1',
@@ -871,8 +952,10 @@ VALID_NODE_JSON = [
      'pm_password': 'p@$$w0rd',
      'pm_port': 1234,
      'ipmi_priv_level': 'USER',
-     'mac': ['dd:ee:ff:aa:bb:cc',
-             '44:55:66:11:22:33'],
+     'ports': [
+         {'address': 'dd:ee:ff:aa:bb:cc'},
+         {'address': '44:55:66:11:22:33'}
+     ],
      'name': 'foobar2',
      'capabilities': {'foo': 'bar'},
      'kernel_id': 'kernel1',
@@ -881,7 +964,9 @@ VALID_NODE_JSON = [
      'pm_addr': '1.2.3.4',
      'pm_user': 'root',
      'pm_password': 'p@$$w0rd',
-     'mac': ['22:22:22:22:22:22'],
+     'ports': [
+         {'address': '22:22:22:22:22:22'}
+     ],
      'capabilities': 'foo:bar,foo1:bar1',
      'cpu': 2,
      'memory': 1024,
@@ -932,7 +1017,9 @@ class TestValidateNodes(base.TestCase):
              'pm_addr': '1.1.1.1',
              'pm_user': 'root',
              'pm_password': 'p@$$w0rd',
-             'mac': ['42']},
+             'ports': [
+                 {'address': '42'}]
+             },
         ]
         self.assertRaisesRegex(exception.InvalidNode,
                                'MAC address 42 is invalid',
@@ -944,12 +1031,16 @@ class TestValidateNodes(base.TestCase):
              'pm_addr': '1.1.1.1',
              'pm_user': 'root',
              'pm_password': 'p@$$w0rd',
-             'mac': ['11:22:33:44:55:66']},
+             'ports': [
+                 {'address': '11:22:33:44:55:66'}
+             ]},
             {'pm_type': 'ipmi',
              'pm_addr': '1.2.1.1',
              'pm_user': 'user',
              'pm_password': 'p@$$w0rd',
-             'mac': ['11:22:33:44:55:66']},
+             'ports': [
+                 {'address': '11:22:33:44:55:66'}
+             ]},
         ]
         self.assertRaisesRegex(exception.InvalidNode,
                                'MAC 11:22:33:44:55:66 is not unique',
