@@ -28,28 +28,48 @@ LOG = logging.getLogger(__name__)
 _KNOWN_INTERFACE_FIELDS = [
     '%s_interface' % field for field in ('boot', 'console', 'deploy',
                                          'inspect', 'management', 'network',
-                                         'power', 'raid', 'storage', 'vendor')
+                                         'power', 'raid', 'rescue', 'storage',
+                                         'vendor')
 ]
 
 CTLPLANE_NETWORK = 'ctlplane'
 
 
+def convert_nodes_json_mac_to_ports(nodes_json):
+    for node in nodes_json:
+        if node.get('mac'):
+            LOG.warning('Key mac is deprecated, please use ports.')
+            for address in node['mac']:
+                try:
+                    node['ports'].append({'address': address})
+                except KeyError:
+                    node['ports'] = [{'address': address}]
+            del node['mac']
+
+    return nodes_json
+
+
 class DriverInfo(object):
     """Class encapsulating field conversion logic."""
     DEFAULTS = {}
 
     def __init__(self, prefix, mapping, deprecated_mapping=None,
-                 mandatory_fields=(), default_port=None):
+                 mandatory_fields=(), default_port=None, hardware_type=None):
         self._prefix = prefix
         self._mapping = mapping
         self._deprecated_mapping = deprecated_mapping or {}
         self._mandatory_fields = mandatory_fields
         self._default_port = default_port
+        self._hardware_type = hardware_type
 
     @property
     def default_port(self):
         return self._default_port
 
+    @property
+    def hardware_type(self):
+        return self._hardware_type
+
     def convert_key(self, key):
         if key in self._mapping:
             return self._mapping[key]
@@ -101,7 +121,7 @@ class DriverInfo(object):
 class PrefixedDriverInfo(DriverInfo):
     def __init__(self, prefix, deprecated_mapping=None,
                  has_port=False, address_field='address',
-                 default_port=None):
+                 default_port=None, hardware_type=None):
         mapping = {
             'pm_addr': '%s_%s' % (prefix, address_field),
             'pm_user': '%s_username' % prefix,
@@ -118,6 +138,7 @@ class PrefixedDriverInfo(DriverInfo):
             deprecated_mapping=deprecated_mapping,
             mandatory_fields=mandatory_fields,
             default_port=default_port,
+            hardware_type=hardware_type,
         )
 
     def unique_id_from_fields(self, fields):
@@ -164,6 +185,7 @@ class RedfishDriverInfo(DriverInfo):
             'redfish', mapping,
             deprecated_mapping=None,
             mandatory_fields=mandatory_fields,
+            hardware_type='redfish',
         )
 
     def _build_id(self, address, system):
@@ -196,6 +218,7 @@ class oVirtDriverInfo(DriverInfo):
         super(oVirtDriverInfo, self).__init__(
             'ovirt', mapping,
             mandatory_fields=list(mapping),
+            hardware_type='staging-ovirt',
         )
 
     def unique_id_from_fields(self, fields):
@@ -212,38 +235,14 @@ class oVirtDriverInfo(DriverInfo):
             return
 
 
-class SshDriverInfo(DriverInfo):
-    DEFAULTS = {'ssh_virt_type': 'virsh'}
-
-    def __init__(self):
-        super(SshDriverInfo, self).__init__(
-            'ssh',
-            {
-                'pm_addr': 'ssh_address',
-                'pm_user': 'ssh_username',
-                # TODO(dtantsur): support ssh_key_filename as well
-                'pm_password': 'ssh_key_contents',
-            },
-            deprecated_mapping={
-                'pm_virt_type': 'ssh_virt_type',
-            },
-            mandatory_fields=['pm_addr', 'pm_user', 'pm_password'],
-        )
-
-    def validate(self, node):
-        super(SshDriverInfo, self).validate(node)
-        if not node.get('mac'):
-            raise exception.InvalidNode(
-                'Nodes with SSH drivers require at least one MAC')
-
-
 class iBootDriverInfo(PrefixedDriverInfo):
     def __init__(self):
         super(iBootDriverInfo, self).__init__(
             'iboot', has_port=True,
             deprecated_mapping={
                 'pm_relay_id': 'iboot_relay_id',
-            }
+            },
+            hardware_type='staging-iboot',
         )
 
     def unique_id_from_fields(self, fields):
@@ -267,23 +266,35 @@ class iBootDriverInfo(PrefixedDriverInfo):
 DRIVER_INFO = {
     # production drivers
     '(ipmi|.*_ipmitool)': PrefixedDriverInfo('ipmi', has_port=True,
-                                             default_port=623),
-    '(idrac|.*_drac)': PrefixedDriverInfo('drac', has_port=True),
-    '(ilo|.*_ilo)': PrefixedDriverInfo('ilo', has_port=True),
-    '(cisco\-ucs\-managed|.*_ucs)': PrefixedDriverInfo('ucs'),
-    '(irmc|.*_irmc)': PrefixedDriverInfo('irmc', has_port=True),
+                                             default_port=623,
+                                             hardware_type='ipmi'),
+    '(idrac|.*_drac)': PrefixedDriverInfo('drac', has_port=True,
+                                          hardware_type='idrac'),
+    '(ilo|.*_ilo)': PrefixedDriverInfo('ilo', has_port=True,
+                                       hardware_type='ilo'),
+    '(cisco\-ucs\-managed|.*_ucs)': PrefixedDriverInfo(
+        'ucs', hardware_type='cisco-ucs-managed'),
+    '(irmc|.*_irmc)': PrefixedDriverInfo('irmc', has_port=True,
+                                         hardware_type='irmc'),
     'redfish': RedfishDriverInfo(),
     # test drivers
     'staging\-ovirt': oVirtDriverInfo(),
-    '.*_iboot': iBootDriverInfo(),
-    '.*_wol': DriverInfo(
+    '(staging\-iboot|.*_iboot)': iBootDriverInfo(),
+    '(staging\-wol|.*wol)': DriverInfo(
         'wol',
         mapping={
             'pm_addr': 'wol_host',
             'pm_port': 'wol_port',
-        }),
-    '.*_amt': PrefixedDriverInfo('amt'),
-    'fake(|_pxe|_agent)': DriverInfo('fake', mapping={}),
+        },
+        hardware_type='staging-wol'),
+    '(staging\-amt|.*_amt)': PrefixedDriverInfo('amt',
+                                                hardware_type='staging-amt'),
+    # fake_pxe was used when no management interface was supported, now
+    # manual-management is used for the same purpose
+    '(manual\-management|fake_pxe|fake_agent)': DriverInfo(
+        'fake', mapping={}, hardware_type='manual-management'),
+    '^fake(|\-hardware)$': DriverInfo('fake', mapping={},
+                                      hardware_type='fake-hardware'),
 }
 
 
@@ -312,8 +323,10 @@ def register_ironic_node(node, client):
 
     if "kernel_id" in node:
         driver_info["deploy_kernel"] = node["kernel_id"]
+        driver_info["rescue_kernel"] = node["kernel_id"]
     if "ramdisk_id" in node:
         driver_info["deploy_ramdisk"] = node["ramdisk_id"]
+        driver_info["rescue_ramdisk"] = node["ramdisk_id"]
 
     interface_fields = {field: node.pop(field)
                         for field in _KNOWN_INTERFACE_FIELDS
@@ -342,7 +355,14 @@ def register_ironic_node(node, client):
             caps = dict_to_capabilities(caps)
         properties.update({"capabilities": six.text_type(caps)})
 
-    create_map = {"driver": node["pm_type"],
+    driver = node['pm_type']
+    if handler.hardware_type and handler.hardware_type != driver:
+        LOG.warning('Replacing deprecated driver %(old)s with the '
+                    'hardware type %(new)s, please update your inventory',
+                    {'old': driver, 'new': handler.hardware_type})
+        driver = handler.hardware_type
+
+    create_map = {"driver": driver,
                   "properties": properties,
                   "driver_info": driver_info,
                   "resource_class": resource_class}
@@ -356,9 +376,14 @@ def register_ironic_node(node, client):
     LOG.debug('Registering node %s with ironic.', node_id)
     ironic_node = client.node.create(**create_map)
 
-    for mac in node.get("mac", []):
-        client.port.create(address=mac, physical_network=CTLPLANE_NETWORK,
-                           node_uuid=ironic_node.uuid)
+    for port in node.get('ports', []):
+        LOG.debug('Creating Bare Metal port for node: %s, with properties: %s.'
+                  % (ironic_node.uuid, port))
+        client.port.create(
+            address=port.get('address'),
+            physical_network=port.get('physical_network', 'ctlplane'),
+            local_link_connection=port.get('local_link_connection'),
+            node_uuid=ironic_node.uuid)
 
     validation = client.node.validate(ironic_node.uuid)
     if not validation.power['result']:
@@ -388,9 +413,9 @@ def _populate_node_mapping(client):
 
 def _get_node_id(node, handler, node_map):
     candidates = set()
-    for mac in node.get('mac', []):
+    for port in node.get('ports', []):
         try:
-            candidates.add(node_map['mac'][mac.lower()])
+            candidates.add(node_map['mac'][port['address'].lower()])
         except KeyError:
             pass
 
@@ -420,8 +445,10 @@ _NON_DRIVER_FIELDS = {'cpu': '/properties/cpus',
                       'root_device': '/properties/root_device',
                       'name': '/name',
                       'resource_class': '/resource_class',
-                      'kernel_id': '/driver_info/deploy_kernel',
-                      'ramdisk_id': '/driver_info/deploy_ramdisk',
+                      'kernel_id': ['/driver_info/deploy_kernel',
+                                    '/driver_info/rescue_kernel'],
+                      'ramdisk_id': ['/driver_info/deploy_ramdisk',
+                                     '/driver_info/rescue_ramdisk'],
                       'capabilities': '/properties/capabilities'}
 
 _NON_DRIVER_FIELDS.update({field: '/%s' % field
@@ -437,9 +464,14 @@ def _update_or_register_ironic_node(node, node_map, client):
                  node_uuid)
 
         patched = {}
-        for field, path in _NON_DRIVER_FIELDS.items():
+        for field, paths in _NON_DRIVER_FIELDS.items():
+            if isinstance(paths, six.string_types):
+                paths = [paths]
+
             if field in node:
-                patched[path] = node.pop(field)
+                value = node.pop(field)
+                for path in paths:
+                    patched[path] = value
 
         driver_info = handler.convert(node)
         for key, value in driver_info.items():
@@ -527,15 +559,16 @@ def validate_nodes(nodes_list):
         except exception.InvalidNode as exc:
             failures.append((index, exc))
 
-        for mac in node.get('mac', ()):
-            if not netutils.is_valid_mac(mac):
-                failures.append((index, 'MAC address %s is invalid' % mac))
+        for port in node.get('ports', ()):
+            if not netutils.is_valid_mac(port['address']):
+                failures.append((index, 'MAC address %s is invalid' %
+                                 port['address']))
 
-            if mac in macs:
+            if port['address'] in macs:
                 failures.append(
-                    (index, 'MAC %s is not unique' % mac))
+                    (index, 'MAC %s is not unique' % port['address']))
             else:
-                macs.add(mac)
+                macs.add(port['address'])
 
         unique_id = handler.unique_id_from_fields(node)
         if unique_id:
@@ -569,7 +602,7 @@ def validate_nodes(nodes_list):
         for field in node:
             converted = handler.convert_key(field)
             if (converted is None and field not in _NON_DRIVER_FIELDS and
-                    field not in ('mac', 'pm_type')):
+                    field not in ('ports', 'pm_type')):
                 failures.append((index, 'Unknown field %s' % field))
 
     if failures:
@@ -634,7 +667,10 @@ def generate_hostmap(baremetal_client, compute_client):
         for port in baremetal_client.port.list(node=bm_node.uuid):
             hostmap[port.address] = {"compute_name": node.name,
                                      "baremetal_name": bm_node.name}
-    return hostmap
+    if hostmap == {}:
+        return None
+    else:
+        return hostmap
 
 
 def run_nova_cell_v2_discovery():
@@ -657,3 +693,15 @@ def get_node_profile(node):
         return capabilities_dict['profile']
 
     return None
+
+
+def get_node_hint(node):
+    """Return the 'capabilities:node' hint associated with the node """
+
+    capabilities = node.get('properties').get('capabilities')
+    capabilities_dict = capabilities_to_dict(capabilities)
+
+    if 'node' in capabilities_dict:
+        return capabilities_dict['node']
+
+    return None
